use crate::{schema, UIDCError};
use microrm::prelude::*;
use ring::signature::{Ed25519KeyPair, KeyPair};
use sha2::Digest;

#[derive(Debug)]
pub enum KeyError {
    Plain(&'static str),
}

pub struct KeyStore<'a, 'r> {
    qi: &'r microrm::QueryInterface<'a>,
}

impl<'a, 'r> KeyStore<'a, 'r> {
    pub fn new(qi: &'r microrm::QueryInterface<'a>) -> Self {
        Self {
            qi
        }
    }

    pub fn generate_in(&self, realm_id: schema::RealmID) -> Result<String, UIDCError> {
        let mut rng = ring::rand::SystemRandom::new();
        let sign_generated = Ed25519KeyPair::generate_pkcs8(&mut rng).map_err(|_| KeyError::Plain("failed to generate key"))?;

        let keydata = sign_generated.as_ref().to_owned();

        let loaded_key = Ed25519KeyPair::from_pkcs8(keydata.as_slice()).expect("couldn't load just-generated key");
        let pubkey = loaded_key.public_key();

        let mut key_hasher = sha2::Sha256::new();
        key_hasher.update(&pubkey.as_ref());
        let mut key_id = base64::encode(key_hasher.finalize());
        key_id.truncate(16);

        self.qi.add(&schema::Key {
            realm: realm_id,
            key_id: key_id.clone(),
            keydata,
        })?;

        Ok(key_id)
    }
}

pub fn inspect(db: &microrm::DB, realm_id: schema::RealmID) -> Result<(), UIDCError> {
    let qi = db.query_interface();
    println!("Keystore loaded.");

    println!("Retrieving keys realm...");
    let keys = qi.get().by(schema::Key::Realm, &realm_id).all()?;

    for key in keys {
        println!("- [{:20}]", key.key_id);
    }
    Ok(())
}

pub fn generate(db: &microrm::DB, realm_id: schema::RealmID) -> Result<(), UIDCError> {
    let qi = db.query_interface();
    let cs = KeyStore::new(&qi);
    match cs.generate_in(realm_id) {
        Ok(_) => (),
        Err(e) => {
            println!("Failed to generate key: {}", e);
        }
    }
    Ok(())
}
