use crate::{schema, user, UIDCError};
use microrm::prelude::*;

pub fn list(qi: &microrm::QueryInterface, realm_id: schema::RealmID) -> Result<(), UIDCError> {
    let users = qi.get().by(schema::User::Realm, &realm_id).all()?;

    println!("User list ({} users):", users.len());

    for user in &users {
        println!("- {:20}", user.username);
        let auth_challenges = qi.get().by(schema::AuthChallenge::User, &user.id()).all()?;
        for ch in &auth_challenges {
            println!("    - Has {:?} authentication challenge", ch.challenge_type);
        }
    }

    Ok(())
}

pub fn create(
    qi: &microrm::QueryInterface,
    realm_id: schema::RealmID,
    username: &str,
) -> Result<(), UIDCError> {
    // check that the user doesn't exist already
    let existing_user = qi
        .get()
        .by(schema::User::Realm, &realm_id)
        .by(schema::User::Username, &username)
        .one()?;

    if existing_user.is_some() {
        log::error!(
            "Can't create user {} in realm, as a user with that username already exists",
            username,
        );
        return Ok(());
    }

    qi.add(&schema::User {
        realm: realm_id,
        username: username.to_owned(),
    })?;
    Ok(())
}

pub fn change_auth(
    qi: &microrm::QueryInterface,
    realm_id: schema::RealmID,
    username: &str,
    change_password: bool,
) -> Result<(), UIDCError> {
    // check that the user exists
    let existing_user = qi
        .get()
        .by(schema::User::Realm, &realm_id)
        .by(schema::User::Username, &username)
        .one()?
        .ok_or(UIDCError::Abort("no such user"))?;

    let user = user::User::from_model(existing_user);

    if change_password {
        let raw_pass = rpassword::prompt_password("Enter new user password: ").unwrap();
        user.set_new_password(&qi, raw_pass.as_bytes());
    }
    Ok(())
}

pub fn inspect(
    qi: &microrm::QueryInterface,
    realm_id: schema::RealmID,
    username: &str,
) -> Result<(), UIDCError> {
    let user = qi
        .get()
        .by(schema::User::Realm, &realm_id)
        .by(schema::User::Username, username)
        .one()?;

    if let Some(user) = user {
        println!("User found: {}", username);
        println!("Groups:");
        for group_membership in qi
            .get()
            .by(schema::GroupMembership::User, &user.id())
            .all()?
        {
            let group = qi
                .get()
                .by_id(&group_membership.group)
                .one()?
                .ok_or(UIDCError::Abort("reference to nonexistent group"))?;
            println!(" - {}", group.shortname);
        }

        println!("Authentication methods:");

        for challenge in qi.get().by(schema::AuthChallenge::User, &user.id()).all()? {
            println!(" - {:?}", challenge.challenge_type);
        }
    } else {
        println!("No such user {} in realm", username);
    }

    Ok(())
}
